/* rd_spice2.c: read in a spice2 rawfile
 *
 * (c) 1998 D. Jeff Dionne  (original author)
 * (c) 1998 Steve Tell, University of NC at Chapel Hill
 		(improved integration with gwave)
 * This is OpenSource Software, under the GPL V2
 * See the file COPYING for details

Here is a quick and dirty hack to gwave to load the first analysis of
a spice2 rawfile into gwave.  Note that spice2 rawfiles are broken in
the Berkeley distribution of spice2, one has to use my patched version,
which will ship in RedHat 5.2.

This quick hack is just a chunk of the simple test code I wrote to check
rawfile generation, and it shows.  It does only the essential stuff, and
throws away most of the header info as well as anything but the first
analysis.  Right now, anything other than a transient run just loads garbage.

 */

#include <stdio.h>
#include <math.h>
#include <string.h>
#include <errno.h>
#include <glib.h>
#include "reader.h"
#include "rawspice.h"


static DataFile *
sp2_read_header (char *name, FILE *fp)
{
	DataFile *df;
	int i;
	int dvsize = 128;
	char buf[80];
	int acan;
	spice_hdr_t hdr;
	spice_var_name_t vname;
	spice_var_type_t vtype;
	spice_var_loc_t vloc;
	spice_plot_title_t title;
	spice_var_t val;

	fread (&val,sizeof(val),1,fp);
	if (memcmp(&val,SPICE_MAGIC,8)) {
		if(analog_read_debug)
			fprintf(stderr, "%s: not a spice rawfile (bad magic number)", name);
		return NULL;
	}

	df = g_new0(DataFile, 1);
	df->filename = g_strdup(name);

	fread (&hdr,sizeof(hdr),1,fp);
	if(analog_read_debug)
		printf ("sp2_read_header: nvars=%d const=%d analysis mode %d",
			hdr.nvars, hdr.const4, hdr.mode);

	acan = ((hdr.mode & 0xc) == 0xc);

	fread (&vname,sizeof(vname),1,fp);
	vname.name[7] = 0;
	df->iv = g_new0(IVar, 1);
	an_init_dataset((DataSet *)df->iv, df, vname.name, TIME);
	df->dv = g_new0(DVar*, dvsize);
	df->ndv = 0;

	for (i=1;i<hdr.nvars;i++) {
		DVar *dv;

		fread (&vname,sizeof(vname),1,fp);
		vname.name[7] = 0;

		if(df->ndv >= dvsize) {
			dvsize *= 2;
			df->dv = g_realloc(df->dv, dvsize * sizeof(DVar*));
		}
		dv = g_new0(DVar, 1);
		dv->iv = df->iv;
		df->dv[df->ndv++] = dv;
		an_init_dataset((DataSet *)&dv->d, df, vname.name, VOLTAGE);
	}

	for (i=0;i<hdr.nvars;i++) {
		fread (&vtype,sizeof(vtype),1,fp);
	}

	for (i=0;i<hdr.nvars;i++) {
		fread (&vloc,sizeof(vloc),1,fp);
	}

	fread (&title,sizeof(title),1,fp);
	strncpy(buf,title.title,sizeof(title));
	buf[sizeof(title)] = 0;
	if(analog_read_debug)
		printf("%s\n",buf);

	return df;
}

static void
sp2_read_vars(FILE *fp, DataFile *df)
{
	int i;
	int nrows = 0;
	spice_var_t val;

	while (1) {
		if (!fread (&val,sizeof(val),1, fp)) return;
		if (memcmp(&val,SPICE_MAGIC,8) == 0) return;
		
		if(ds_blockno(nrows) >= df->iv->d.bpused) {
			an_expand_dset((DataSet *)df->iv);

			for(i = 0; i < df->ndv; i++)
				an_expand_dset((DataSet *)df->dv[i]);
		}
		an_set_point((DataSet *)&df->iv->d, nrows, val.val);
		df->iv->d.nvalues++;
		
		for(i = 0; i < df->ndv; i++) {
			fread (&val,sizeof(val),1,fp);
			an_set_point((DataSet *)&df->dv[i]->d, nrows, val.val);
			df->dv[i]->d.nvalues++;
		}
		nrows++;
	}
}

/*
 * Read a whole file into memory.
 *   Returns pointer to DataFile on success, NULL on failure
 * TODO: figure out a way to return more information on failure.
 */
DataFile *
sp2_read_file(char *name, FILE *fp)
{
	DataFile *df;

	df = sp2_read_header(name, fp);
	if(!df)
		return NULL;
	sp2_read_vars(fp,df);
	return df;
}
